@@ -210,6 +210,13 @@ impl<const N: usize> Decimal<N> {
         self.exponent = exponent;
     }
 
+    /// Returns access to the valid digits from `self`'s `lsu`.
+    pub(crate) fn lsu(&self) -> &[u16] {
+        let valid_lsu_len = (usize::try_from(self.digits()).unwrap() + decnumber_sys::DECDPUN - 1)
+            / decnumber_sys::DECDPUN;
+        &self.lsu[0..valid_lsu_len]
+    }
+
     /// Reports whether the number is finite.
     ///
     /// A finite number is one that is neither infinite nor a NaN.
@@ -309,8 +316,28 @@ impl<const N: usize> Decimal<N> {
     /// Returns the raw parts of this decimal.
     ///
     /// The meaning of these parts are unspecified and subject to change.
-    pub fn to_raw_parts(&self) -> (u32, i32, u8, [u16; N]) {
-        (self.digits, self.exponent, self.bits, self.lsu)
+    pub fn to_raw_parts(&self) -> (u32, i32, u8, &[u16]) {
+        (self.digits, self.exponent, self.bits, &self.lsu())
+    }
+
+    /// Returns a `Decimal::<N>` with the supplied raw parts.
+    ///
+    /// # Safety
+    ///
+    /// The raw parts must be valid according to the guarantees required by the
+    /// underlying C library, or undefined behavior can result. The easiest way
+    /// to uphold these guarantees is to ensure the raw parts originate from a
+    /// call to `Decimal::to_raw_parts`.
+    pub unsafe fn from_raw_parts(digits: u32, exponent: i32, bits: u8, lsu_in: &[u16]) -> Self {
+        let mut lsu = [0; N];
+        lsu[..lsu_in.len()].copy_from_slice(lsu_in);
+
+        Decimal::<N> {
+            digits,
+            exponent,
+            bits,
+            lsu,
+        }
     }
 
     /// Returns a string of the number in standard notation, i.e. guaranteed to
@@ -318,6 +345,14 @@ impl<const N: usize> Decimal<N> {
     pub fn to_standard_notation_string(&self) -> String {
         to_standard_notation_string!(self)
     }
+
+    /// Removes insignificant trailing zeros from a number, unconditionally, and
+    /// stores the modified value in `self`.
+    pub fn trim(&mut self) {
+        unsafe {
+            decnumber_sys::decNumberTrim(self.as_mut_ptr());
+        }
+    }
 }
 
 impl<const N: usize> Default for Decimal<N> {
@@ -522,6 +557,79 @@ impl<const N: usize> TryFrom<Decimal<N>> for isize {
     }
 }
 
+macro_rules! decnum_tryinto_primitive_float {
+    ($p:ty, $cx:expr, $d:expr) => {{
+        if $d.is_infinite() {
+            return Ok(if $d.is_negative() {
+                <$p>::NEG_INFINITY
+            } else {
+                <$p>::INFINITY
+            });
+        }
+        if $d.is_nan() {
+            return Ok(<$p>::NAN);
+        }
+
+        const TEN: $p = 10.0;
+        const DECDPUN_F: $p = decnumber_sys::DECDPUN as $p;
+
+        let mut e = $d.exponent() as $p;
+        let mut f: $p = 0.0;
+        for u in $d.coefficient_units() {
+            // `powi` gives wrong results on some input, whereas `powf` does not
+            f += <$p>::from(*u) * TEN.powf(e);
+            e += DECDPUN_F;
+        }
+        if $d.is_negative() {
+            f = -f;
+        }
+
+        // Value over- or underflows $p:
+        // - f.is_infinite() represents generic overflow
+        // - f.is_nan() can occur when multiplying a coefficient unit by a power
+        //   of 10 that exceeds the primitive type's maximum exponent
+        // - (!$d.is_zero() && f == 0.0) represents underflow
+        if f.is_infinite() || f.is_nan() || (!$d.is_zero() && f == 0.0) {
+            let mut s = $cx.status();
+            s.set_invalid_operation();
+            $cx.set_status(s);
+            Err(TryFromDecimalError)
+        } else {
+            Ok(f)
+        }
+    }};
+}
+
+impl<const N: usize> TryFrom<Decimal<N>> for f32 {
+    type Error = TryFromDecimalError;
+    fn try_from(n: Decimal<N>) -> Result<f32, Self::Error> {
+        let mut cx = Context::<Decimal<N>>::default();
+        cx.try_into_f32(n)
+    }
+}
+
+impl<const N: usize> TryFrom<Decimal<N>> for f64 {
+    type Error = TryFromDecimalError;
+    fn try_from(n: Decimal<N>) -> Result<f64, Self::Error> {
+        let mut cx = Context::<Decimal<N>>::default();
+        cx.try_into_f64(n)
+    }
+}
+
+impl<const N: usize> From<f32> for Decimal<N> {
+    fn from(n: f32) -> Decimal<N> {
+        let mut cx = Context::<Decimal<N>>::default();
+        cx.from_f32(n)
+    }
+}
+
+impl<const N: usize> From<f64> for Decimal<N> {
+    fn from(n: f64) -> Decimal<N> {
+        let mut cx = Context::<Decimal<N>>::default();
+        cx.from_f64(n)
+    }
+}
+
 impl<const N: usize> From<u32> for Decimal<N> {
     fn from(n: u32) -> Decimal<N> {
         let mut d = Decimal::default();
@@ -736,14 +844,13 @@ impl<const N: usize> Context<Decimal<N>> {
     {
         validate_n(N);
         let c_string = CString::new(s).map_err(|_| ParseDecimalError)?;
-        let mut d = MaybeUninit::<Decimal<N>>::uninit();
-        let d = unsafe {
+        let mut d = Decimal::zero();
+        unsafe {
             decnumber_sys::decNumberFromString(
                 d.as_mut_ptr() as *mut decnumber_sys::decNumber,
                 c_string.as_ptr(),
                 &mut self.inner,
             );
-            d.assume_init()
         };
         if (self.inner.status & decnumber_sys::DEC_Conversion_syntax) != 0 {
             Err(ParseDecimalError)
@@ -1003,6 +1110,48 @@ impl<const N: usize> Context<Decimal<N>> {
         decnum_tryinto_primitive_uint!(u128, self, 39, d)
     }
 
+    /// Attempts to convert `d` to `f32` or fails if not possible.
+    ///
+    /// Note that this function:
+    /// - Errors for values that over- or underflow `f32`, rather than returning
+    ///   infinity or `0.0`, respectively.
+    /// - Returns a primitive infinity or NaN if `d` is an equivalent value.
+    pub fn try_into_f32(&mut self, d: Decimal<N>) -> Result<f32, TryFromDecimalError> {
+        decnum_tryinto_primitive_float!(f32, self, d)
+    }
+
+    /// Attempts to convert `d` to `f32` or fails if not possible.
+    ///
+    /// Refer to the comments on [`Self::try_into_f32()`], which also apply to this
+    /// function.
+    pub fn try_into_f64(&mut self, d: Decimal<N>) -> Result<f64, TryFromDecimalError> {
+        decnum_tryinto_primitive_float!(f64, self, d)
+    }
+
+    /// Converts an `f32` to a `Decimal<N>`.
+    ///
+    /// Note that this conversion is infallible because `f32`'s:
+    /// - Maximum precision is ~8
+    /// - Min/max exponent is ~ -37, 37
+    ///
+    /// Both of these are guaranteed to fit comfortably within `Decimal`'s
+    /// constraints.
+    pub fn from_f32(&mut self, n: f32) -> Decimal<N> {
+        self.parse(n.to_string().as_str()).unwrap()
+    }
+
+    /// Converts an `f64` to a `Decimal<N>`.
+    ///
+    /// Note that this conversion is infallible because `f64`'s:
+    /// - Maximum precision is ~18
+    /// - Min/max exponent is ~ -305, 305
+    ///
+    /// Both of these are guaranteed to fit comfortably within `Decimal`'s
+    /// constraints.
+    pub fn from_f64(&mut self, n: f64) -> Decimal<N> {
+        self.parse(n.to_string().as_str()).unwrap()
+    }
+
     /// Computes the digitwise logical inversion of `n`, storing the result in
     /// `n`.
     pub fn invert(&mut self, n: &mut Decimal<N>) {
@@ -1277,6 +1426,14 @@ impl<const N: usize> Context<Decimal<N>> {
         }
     }
 
+    /// Rounds the number to an integral value using the rounding mode in the
+    /// context.
+    pub fn round(&mut self, n: &mut Decimal<N>) {
+        unsafe {
+            decnumber_sys::decNumberToIntegralExact(n.as_mut_ptr(), n.as_ptr(), &mut self.inner);
+        }
+    }
+
     /// Shifts the digits of `lhs` by `rhs`, storing the result in `lhs`.
     ///
     /// If `rhs` is positive, shifts to the left. If `rhs` is negative, shifts
